

parse.progparams <- function(progparam.lines) {
  tokens <- strsplit(progparam.lines, split="[[:blank:]]+")
  
  linenum <- 1
  stopifnot(tokens[[linenum]][1] == "beast.popmean.lnprior")
  stopifnot(length(tokens[[linenum]]) == 3)
  beast.popmean.lnprior.meanlog <- as.double(tokens[[linenum]][2])
  beast.popmean.lnprior.sdlog <- as.double(tokens[[linenum]][3])
  stopifnot(is.finite(beast.popmean.lnprior.meanlog))
  stopifnot(is.finite(beast.popmean.lnprior.sdlog))
  
  linenum <- 2
  stopifnot(tokens[[linenum]][1] == "beast.growthrate.lnprior")
  stopifnot(length(tokens[[linenum]]) == 3)
  beast.growthrate.lnprior.meanlog <- as.double(tokens[[linenum]][2])
  beast.growthrate.lnprior.sdlog <- as.double(tokens[[linenum]][3])
  stopifnot(is.finite(beast.growthrate.lnprior.meanlog))
  stopifnot(is.finite(beast.growthrate.lnprior.sdlog))
  
  linenum <- 3
  stopifnot(tokens[[linenum]][1] == "beast.relativerates.lnprior")
  stopifnot(length(tokens[[linenum]]) == 3)
  beast.relativerates.lnprior.meanlog <- as.double(tokens[[linenum]][2])
  beast.relativerates.lnprior.sdlog <- as.double(tokens[[linenum]][3])
  stopifnot(is.finite(beast.relativerates.lnprior.meanlog))
  stopifnot(is.finite(beast.relativerates.lnprior.sdlog))
  
  linenum <- 4
  stopifnot(tokens[[linenum]][1] == "beast.collapseweight.betaprior")
  stopifnot(length(tokens[[linenum]]) == 3)
  beast.collapseweight.betaprior.shapeA <- as.double(tokens[[linenum]][2])
  beast.collapseweight.betaprior.shapeB <- as.double(tokens[[linenum]][3])
  stopifnot(is.finite(beast.collapseweight.betaprior.shapeA))
  stopifnot(is.finite(beast.collapseweight.betaprior.shapeB))  
  
  beast.priors <- list(popmean.meanlog=beast.popmean.lnprior.meanlog,
                       popmean.sdlog=beast.popmean.lnprior.sdlog,
                       growthrate.meanlog=beast.growthrate.lnprior.meanlog,
                       growthrate.sdlog=beast.growthrate.lnprior.sdlog,
                       relativerates.meanlog=beast.relativerates.lnprior.meanlog,
                       relativerates.sdlog=beast.relativerates.lnprior.sdlog,
                       collapseweight.shape=beast.collapseweight.betaprior.shapeA,
                       collapseweight.shapeB=beast.collapseweight.betaprior.shapeB)
                       
                       
  linenum <- 5
  stopifnot(tokens[[linenum]][1] == "beast.collapseheight")
  stopifnot(length(tokens[[linenum]]) == 2)
  beast.collapseheight <- as.double(tokens[[linenum]][2])                       
  
  linenum <- 6
  stopifnot(tokens[[linenum]][1] == "beastseed")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastseed <- as.integer(tokens[[linenum]][2])

  linenum <- 7
  stopifnot(tokens[[linenum]][1] == "beastchainlength")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastchainlength <- as.integer(tokens[[linenum]][2])

  linenum <- 8
  stopifnot(tokens[[linenum]][1] == "beastscreen.logevery")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastscreen.logevery <- as.integer(tokens[[linenum]][2])

  linenum <- 9
  stopifnot(tokens[[linenum]][1] == "beastparams.logevery")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastparams.logevery <- as.integer(tokens[[linenum]][2])

  linenum <- 10
  stopifnot(tokens[[linenum]][1] == "beastgtrees.logevery")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastgtrees.logevery  <- as.integer(tokens[[linenum]][2])

  linenum <- 11
  stopifnot(tokens[[linenum]][1] == "beastspptree.logevery")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastspptree.logevery <- as.integer(tokens[[linenum]][2])

  linenum <- 12
  stopifnot(tokens[[linenum]][1] == "treeann.gtrees.burnin")
  stopifnot(length(tokens[[linenum]]) == 2)
  treeann.gtrees.burnin  <- as.integer(tokens[[linenum]][2])

  linenum <- 13
  stopifnot(tokens[[linenum]][1] == "treeann.stree.burnin")
  stopifnot(length(tokens[[linenum]]) == 2)
  treeann.stree.burnin <- as.integer(tokens[[linenum]][2])
  
  linenum <- 14
  stopifnot(tokens[[linenum]][1] == "sppdelimanal.burnin")
  stopifnot(length(tokens[[linenum]]) == 2)
  sppdelimanal.burnin <- as.integer(tokens[[linenum]][2])
  
  linenum <- 15
  stopifnot(tokens[[linenum]][1] == "sppdelimanal.collapseheight")
  stopifnot(length(tokens[[linenum]]) == 2)
  sppdelimanal.collapseheight <- as.double(tokens[[linenum]][2])
  
  linenum <- 16
  stopifnot(tokens[[linenum]][1] == "sppdelimanal.simcutoff")
  stopifnot(length(tokens[[linenum]]) == 2)
  sppdelimanal.simcutoff <- as.integer(tokens[[linenum]][2])
  
  list(beast.priors=beast.priors, beast.collapseheight=beast.collapseheight,
       beastseed=beastseed, beastchainlength=beastchainlength,
       beastscreen.logevery=beastscreen.logevery, beastparams.logevery=beastparams.logevery,
       beastgtrees.logevery=beastgtrees.logevery, beastspptree.logevery=beastspptree.logevery,
       treeann.gtrees.burnin=treeann.gtrees.burnin, treeann.stree.burnin=treeann.stree.burnin,
       sppdelimanal.burnin=sppdelimanal.burnin, sppdelimanal.collapseheight=sppdelimanal.collapseheight,
       sppdelimanal.simcutoff=sppdelimanal.simcutoff)
 }









