

make.spptree <- function(network.cfg) {
  spptree <- as.list(NULL)
  snodes <- network.cfg$sppnodes
  spptree <- snodes2sppsubtree(spptree, snodes, length(snodes))
  # TODO should do some checks: pops equal when they should be?
  spptree
  }                        
  


  
print.spptree <- function(spptree) {
  cladelen <- 30
  alrlen <- cladelen + 9
  dbllen <- 8
  hlen <- alrlen + (dbllen+1)
  
  header.txt <- paste(rgt.pad("idx   clade", cladelen), "anc  l  r", sep="")
  header.txt <- rgt.pad(header.txt, alrlen)
  header.txt <- paste(header.txt, "     hgt")
  header.txt <- rgt.pad(header.txt, hlen)
  header.txt <- paste(header.txt, "nlin")
  header.txt <- paste(header.txt, "  tippop  prevpop")
  cat(header.txt, "\n")
  txt.lens <- list(clade=cladelen, alr=alrlen, hgts=hlen, dbl=dbllen)
  for (i in 1:length(spptree)) {
    print.spptree.node(i, spptree[[i]], txt.lens)
    }
  }  
  



print.spptree.node <- function(i, sn, txt.lens) {
  cladelen <- txt.lens$clade      
  alrlen <- txt.lens$alr
  hlen <- txt.lens$hgts
  dbllen <- txt.lens$dbl
  txt <- paste(lft.pad(i,2), sn$clade)
  txt <- rgt.pad(txt, cladelen)
  txt <- paste(txt, lft.pad(sn$anc,2), 
                  lft.pad(sn$lft,2), lft.pad(sn$rgt,2))
  txt <- rgt.pad(txt, alrlen)
  txt <- paste(txt, lft.pad(sn$nheight, dbllen))
  txt <- rgt.pad(txt, hlen)
  txt <- paste(txt, lft.pad(sn$nlins,4))
  txt <- paste(txt, lft.pad(sn$tippop, dbllen), lft.pad(sn$rootpop, dbllen))
  cat(txt, "\n")
  }


lft.pad <- function(txt, n) {
  while (nchar(txt) < n) {
    txt <- paste("", txt)
    }
  txt
  }

rgt.pad <- function(txt, n) {
  while (nchar(txt) < n) {
    txt <- paste(txt, "")
    }
  txt
  }
 


spptree.to.Newick <- function(spptree) {
root <- -1
for (n in 1:length(spptree)) {
  if (spptree[[n]]$anc < 0)
    {
    stopifnot(root < 0)
    root <- n
    }
  }
stopifnot(root > 0)
paste(sppsubtree.to.Newick(spptree, root), ";", sep="")
}


sppsubtree.to.Newick <- function(spptree, node) {
  lftnode <- spptree[[node]]$lft
  rgtnode <- spptree[[node]]$rgt
  if (lftnode > 0) {
    lftnewick <- sppsubtree.to.Newick(spptree, lftnode)
    rgtnewick <- sppsubtree.to.Newick(spptree, rgtnode)
    lftbtime <- spptree[[node]]$nheight - spptree[[lftnode]]$nheight
    rgtbtime <- spptree[[node]]$nheight - spptree[[rgtnode]]$nheight
    newick <- paste("(", lftnewick, ":", toString(lftbtime), ",",
                         rgtnewick, ":", toString(rgtbtime), ")", sep="")
    } else {
    newick <- spptree[[node]]$clade
    }
  newick
  }



 
snodes2sppsubtree <- function(spptree, snodes, n) {
  snode <- snodes[[n]]
  if (snode$nheight == 0) {
    # tip
    newn <- sppnode.from.snode(snode) 
    spptree <- c(spptree, list(newn))
    } else {
    newn <- sppnode.from.snode(snode)
    sc <- splitclade(snode$clade)
    
    hlft <- snode.of.clade(snodes, sc$lftclade)
    spptree <- snodes2sppsubtree(spptree, snodes, hlft)
    lft <- length(spptree)
    
    hrgt <- snode.of.clade(snodes, sc$rgtclade)
    spptree <- snodes2sppsubtree(spptree, snodes, hrgt)
    rgt <- length(spptree)
    
    spptree <- c(spptree, list(newn))
    here <- length(spptree)
    
    spptree[[lft]]$anc <- here
    spptree[[rgt]]$anc <- here
    spptree[[here]]$lft <- lft   
    spptree[[here]]$rgt <- rgt  
    }
  spptree
  } 
  
  

  
  
sppnode.from.snode <- function(snode) {
  clade <- snode$clade
  list(clade=clade, anc=-1, lft=-1, rgt=-1, 
       nheight=snode$nheight,
       tippop=snode$tippop, rootpop=snode$rootpop, 
       root=snode$root, nlins=snode$nlins)
}


       


dud.sppnode <- function() {
list(clade="", anc=-1, lft=-1, rgt=-1, root=FALSE,
       height=0, nlins=-1, 
       tippop=-1, rootpop=-1)
}


splitclade <- function(clade) {
  x <- strsplit(clade, split="")[[1]]
  stopifnot(x[1]=="(")
  stopifnot(x[length(x)]==")")
  level <- 0
  j <- -1
  for (i in 1:length(x)) {
    if (x[i] == "(") { level <- level + 1 }
    if (x[i] == ")") { level <- level - 1 }
    if (x[i] == ","  &&  level==1) {
      stopifnot(j==-1)
      j <- i
      }
    }
  stopifnot(j>0)
  lftclade <- paste(x[2:(j-1)], collapse="")
  rgtclade <- paste(x[(j+1):(length(x)-1)], collapse="")
  list(lftclade=lftclade, rgtclade=rgtclade) 
  } 


snode.of.clade <- function(snodes, clade) {
  j <- -1
  for (i in 1:length(snodes)) {
    if (snodes[[i]]$clade == clade) {
      stopifnot(j==-1)
      j <- i
      }
    }
  stopifnot(j>0)
  j
  }		
		
		
sppnode.of.clade <- function(spptree, clade) {
  j <- -1
  for (i in 1:length(spptree)) {
    if (spptree[[i]]$clade == clade) {
      stopifnot(j==-1)
      j <- i
      }
    }
  cat("spptree", clade, j, "\n")
  stopifnot(j>0)
  j
  }
