
#################### This is 'exported' ########################################

make.beastxml.from.sim.spptree <- function(nofgenes, taxa, 
                   beast.priors, 
                   seqs.fpathbase, sampledgtrees.fpathbase,
                   sampledspptrees.fpath, sampledparams.fpath, DBUGTUNE.fpath) {
  make.beastxml.start("Simulated data made by seqgen and R code.",
                      "For testing DISSECT.")
  
  totalnofseqs <- make.taxa.and.alignments(nofgenes, seqs.fpathbase)
  commentln("Patterns")
  for (g in 1:nofgenes) {
    make.patterns(g)
    }

  make.beastxml.after.patterns(length(taxa), nofgenes, totalnofseqs,
                   taxa, beast.priors,
                   sampledgtrees.fpathbase, sampledspptrees.fpath, sampledparams.fpath,
                   DBUGTUNE.fpath)
}
  
  
  
################################################################################
####################### rest is 'private' ###################################### 



make.beastxml.start <- function(comment1, comment2) {
  catln(0, "<?xml version=\"1.0\" standalone=\"yes\"?>")
  commentln(comment1)
  commentln(comment2)
  catln(0, "<beast>")
}




make.beastxml.after.patterns <- function(nofminclusters, nofgenes, totalnofseqs,
                   taxa, beast.priors,
                   sampledgtrees.fpathbase, sampledspptrees.fpath, sampledparams.fpath,
                   DBUGTUNE.fpath) {
  
  commentln(" ********************************************************* ")    
  commentln(" **********    gene trees models, priors     ************* ")    
  commentln(" ********************************************************* ") 
   
  commentln("Constant size population parameter used to generate starting trees.")
  make.constantSize()
  commentln("Randomly generated starting trees for genes.")
  for (g in 1:nofgenes) {
    make.coalescentTree(g, 0.02)
    }
  commentln("Tree models for gene trees.")
  for (g in 1:nofgenes) {
    make.treeModel(g)
    }
  commentln("Branch models for gene trees.")
  for (g in 1:nofgenes) {
    make.strictClockBranchRate(g)
    }
  commentln("HKY substitution models for gene trees.")
  for (g in 1:nofgenes) {
    make.HKYModel(g)
    }  
  commentln("Site models for gene trees.")
  for (g in 1:nofgenes) {
    make.siteModel(g)
    }   
  commentln("Likelihoods for gene trees.")
  for (g in 1:nofgenes) {
    make.treeLikelihood(g)
    } 
    

  commentln(" ********************************************************* ")    
  commentln(" **********     Species tree models, priors      ********* ")    
  commentln(" ********************************************************* ") 
   
  commentln("Species bindings. Assignments of individuals to species, and") 
  commentln("references to the gene trees.") 
  make.SpeciesBindings(nofgenes, taxa)
  commentln("Species tree")    
  make.speciesTree(0.02)
  commentln("Birth-death-collapse model for species tree")
  make.birthDeathCollapseModel()
  commentln("Species Tree likelihood for topology and node heights")
  make.speciationLikelihood()
  commentln("Multispecies coalescent likelihood for species tree")
  make.speciesCoalescent()
  commentln("Species tree population size prior: gama2 + gamma4 ")
  make.mixedDistributionLikelihood(nofminclusters, 0.02)
  
  commentln(" ********************************************************* ")    
  commentln("*******************     Statistics      ******************")    
  commentln(" ********************************************************* ")  
      
  make.NClustersStatistic()

  make.trmcaStatistic(taxa, "")
  for (letter in c("a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k")) {
    taxasubset = taxa.subset(letter, taxa)
    if (length(taxasubset) > 0) {
      make.trmcaStatistic(taxasubset, letter)
      }
    }
  
  commentln(" ********************************************************* ")    
  commentln("*******************     Operators      *******************")    
  commentln(" ********************************************************* ")  
      
  make.operators(nofgenes, totalnofseqs)
  
  commentln(" ********************************************************* ")    
  commentln("*******************        mcmc        *******************")    
  commentln(" ********************************************************* ")  
  
  make.mcmc(nofgenes, taxa, beast.priors,
          sampledgtrees.fpathbase, sampledspptrees.fpath, 
          sampledparams.fpath, DBUGTUNE.fpath)

  catln(0, "</beast>")
}

                   
taxa.subset <- function(letter, taxa) {
  txsubset <- character(0)
  for (j in 1:length(taxa)) {
    if (substr(taxa[j], 1, 1) == letter) {
      txsubset <- c(txsubset, taxa[j])
    }
  }
  txsubset
}
           
 
  
  
#taxa.array.to.structure <- function(taxa) {
#  struct <- NULL
#  spp <- collect.unique.string.parts(taxa, 3, 3)
#  for (i in 1:length(spp)) {
#    taxa.of.spp <- NULL
#    for (j in 1:length(taxa)) {
#      if (substr(taxa[j], 3, 3) == spp[i]) {
#        taxa.of.spp <- c(taxa.of.spp, taxa[j])
#      }
#    }
#    struct <- c(struct, list(taxa.of.spp.to.structure(spp[i], taxa.of.spp))) 
#    }
#  struct
#  }
#
#
#taxa.of.spp.to.structure <- function(sp, tofs) {
#  indivs <- collect.unique.string.parts(tofs, 1, 2)
#  sqs <- collect.unique.string.parts(tofs, 4, 4)
#  list(sp=sp, indivs=indivs, sqs=sqs)
#  } 
#
#
#collect.unique.string.parts <- function(strings, beg, last) {
#  parts <- substr(strings[1], beg, last)
#  if (length(strings) > 1) {
#    for (i in 2:length(strings)) {
#      part <- substr(strings[i], beg, last)
#      got = FALSE
#      for (j in 1:length(parts)) {
#        if (part == parts[j]) { got = TRUE }
#        }
#      if (!got) {
#        parts <- c(parts, part)
#        }
#      }
#    }
#  parts
#  } 





  
# Read in the sequences files and join to make some XML
make.taxa.and.alignments <- function(nofgenes, seqs.fpathbase) {
commentln("Taxa. Each taxon is a sequence (labelled A,B,...) from")
commentln("an individual (01,02,03,...) in a species (a,b,c...).")
totalnofseqs <- 0
  for (g in 1:nofgenes) {
    numastext <- as.character(g)
    textlines <- readLines(paste(seqs.fpathbase, numastext, ".txt", sep=""))
    n <- length(textlines)
    totalnofseqs <- totalnofseqs + n
    tokens <- strsplit(textlines, split=" +")
  
    if (g == 1) {
      taxa <- rep("", n-1)
      for (i in 2:n) {
        taxa[i-1] <- tokens[[i]][1]
        }
      taxa <- sort(taxa)
      
      cat("\t<taxa id=\"taxa\">\n")
      for (i in 2:n) {
        cat("\t\t<taxon id=\"", taxa[i-1], "\"/>\n", sep="")
        }
      cat("\t</taxa>\n")
      } else {
      # error check would be good
      }
    commentln(paste("Simulated alignment for gene", g, "of", nofgenes, "genes."))
    cat("\t<alignment id=\"alignment", numastext, "\" dataType=\"nucleotide\">\n", sep="")
    for (i in 2:n) {
      taxon <- tokens[[i]][1]
      seqnce <- tokens[[i]][2]
      cat("\t\t<sequence>\n", sep="")
      cat("\t\t\t<taxon idref=\"", taxon, "\" />\n", sep="")
      cat(seqnce, "\n", sep="")
      cat("\t\t</sequence>\n", sep="")
      }
    cat("\t</alignment>\n")
    }
  totalnofseqs
  }



gene.tree.weight <- function(wt, nofgenes, totalnofseqs) {
  relativeweight <- totalnofseqs / (100*nofgenes)
  as.integer(round(wt+wt*relativeweight))
  }


make.operators <- function(nofgenes, totalnofseqs) { 
  commentln("Operators for gene trees.")
  catln(1, "<operators", id("operators"), ">")
  for (g in 1:nofgenes) {
    make.scaleOperator.kappa(g, weight=gene.tree.weight(1, nofgenes, totalnofseqs))
    }    
  for (g in 1:nofgenes) {
    make.deltaExchange.frequencies(g, weight=gene.tree.weight(1, nofgenes, totalnofseqs))
    }
  for (g in 1:nofgenes) {
    if (g > 1) {
      make.scaleOperator.clockrate(g, weight=gene.tree.weight(1, nofgenes, totalnofseqs))
      }
    }
  for (g in 1:nofgenes) {
    make.subtreeSlide.treeModel(g, weight=gene.tree.weight(15, nofgenes, totalnofseqs)) 
    }
  for (g in 1:nofgenes) {
    make.narrowExchange.treeModel(g, weight=gene.tree.weight(15, nofgenes, totalnofseqs)) 
    }    
  for (g in 1:nofgenes) {
    make.wideExchange.treeModel(g, weight=gene.tree.weight(3, nofgenes, totalnofseqs)) 
    }    
  for (g in 1:nofgenes) {
    make.wilsonBalding.treeModel(g, weight=gene.tree.weight(3, nofgenes, totalnofseqs)) 
    }    
  for (g in 1:nofgenes) {
    make.scaleOperator.treeModel.rootHeight(g, weight=gene.tree.weight(3, nofgenes, totalnofseqs)) 
    }    
  for (g in 1:nofgenes) {
    make.uniformOperator.treeModel.internalNodeHeights(g, weight=gene.tree.weight(30, nofgenes, totalnofseqs)) 
    }    
  # this moves the inode heights plus the clock rates relative 
  # to the first gene - note g==1 case done differently
  for (g in 1:nofgenes) {
    make.upDownOperator.clockrate.allInternalNodeHeights(g, weight=gene.tree.weight(30, nofgenes, totalnofseqs)) 
    }    

  commentln("Operator for both species tree and gene trees, stretches/squeezes everything.")
  make.upDownOperator.clocks.pops.heights(nofgenes, weight=30)
  commentln("Operators for species tree.")
	make.general.scaleOperator(0.9, 5+nofgenes, "species.popMean")	
	make.general.scaleOperator(0.75, 3, "species.birthDeathCollapse.meanGrowthRate")
	make.general.scaleOperator(0.75, 3, "species.birthDeathCollapse.relativeDeathRate")
	make.general.scaleOperator(0.75, 3, "species.birthDeathCollapse.originHeight")
	make.general.scaleOperator(0.75, 3, "species.birthDeathCollapse.collapseWeight")
	make.general.scaleOperator(0.5, 94, "speciesTree.splitPopSize")
  make.nodeReHeightOperator(94)
  catln(1, "</operators>")
  }
  

 
  
make.logs <- function(nofgenes, taxa,
                      sampledgtrees.fpathbase, sampledspptrees.fpath,
                      sampledparams.fpath, DBUGTUNE.fpath) {
  commentln("Log to screen.")
  catln(2, "<log", id("screenLog"), namevalue("logEvery", BEASTXML.screen.logevery), ">") 
  catln(3, "<column", namevalue("label", "Posterior"), namevalue("dp", 4),
                       namevalue("width", 12), ">")
  catln(3, idref("posterior", "posterior"))
  catln(3, "</column>")
  
  catln(3, "<column", namevalue("label", "Prior"), namevalue("dp", 4),
                       namevalue("width", 12), ">")
  catln(3, idref("prior", "prior"))
  catln(3, "</column>")
  
  catln(3, "<column", namevalue("label", "Likelihood"), namevalue("dp", 4),
                       namevalue("width", 12), ">")
  catln(3, idref("likelihood", "likelihood"))
  catln(3, "</column>")
  
  catln(3, "<column", namevalue("label", "nClusters"), namevalue("dp", 0),
                       namevalue("width", 8), ">")
  catln(3, idref("bdcNClustersStatistic", "nClusters"))
  catln(3, "</column>")
  
  catln(3, "<column", namevalue("label", "cllpsWt"), namevalue("dp", 4),
                       namevalue("width", 8), ">")
  catln(3, idref("parameter", "species.birthDeathCollapse.collapseWeight"))
  catln(3, "</column>")
  
  catln(3, "<column", namevalue("label", "PopSF"), namevalue("sf", 6),
                       namevalue("width", 12), ">")
  catln(3, idref("parameter", "species.popMean"))
  catln(3, "</column>")
  catln(2, "</log>")
  
  commentln("Main log to file.")
  catln(2, "<log", id("fileLog"), namevalue("logEvery", BEASTXML.params.logevery), 
            namevalue("fileName", sampledparams.fpath), ">") 
  catln(3, idref("posterior", "posterior"))
  catln(3, idref("prior", "prior"))
  catln(3, idref("likelihood", "likelihood"))
  catln(3, idref("speciesCoalescent", "species.coalescent"))
  catln(3, idref("mixedDistributionLikelihood", "species.popSize"))
  catln(3, idref("speciationLikelihood", "speciation.likelihood"))
  
  catln(3, idref("parameter", "species.popMean"))
  catln(3, idref("parameter", "speciesTree.splitPopSize"))
  catln(3, idref("parameter", "species.birthDeathCollapse.meanGrowthRate"))
  catln(3, idref("parameter", "species.birthDeathCollapse.relativeDeathRate"))
  catln(3, idref("parameter", "species.birthDeathCollapse.originHeight"))
  catln(3, idref("parameter", "species.birthDeathCollapse.collapseWeight"))
  
  catln(3, idref("bdcNClustersStatistic", "nClusters"))
  catln(3, idref("tmrcaStatistic", trmca.idname("")))
  for (letter in c("a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k")) {
    txss <- taxa.subset(letter, taxa)
    if (length(txss) > 0) {
      catln(3, idref("tmrcaStatistic", trmca.idname(letter)))
      }
    }

  for (g in 1:nofgenes) {
    catln(3, numbered.idref("parameter", "", g, ".treeModel.rootHeight"))
    }
  for (g in 1:nofgenes) {
    catln(3, numbered.idref("parameter", "", g, ".kappa"))
    }
  for (g in 1:nofgenes) {
    catln(3, numbered.idref("parameter", "", g, ".frequencies"))
    }
  for (g in 1:nofgenes) {
    catln(3, numbered.idref("parameter", "", g, ".clock.rate"))
    }
  for (g in 1:nofgenes) {
    catln(3, numbered.idref("treeLikelihood", "", g, ".treeLikelihood"))
    }
  catln(2, "</log>")
  
  commentln("Gene tree log files.")
  for (g in 1:nofgenes) {
    fname <- paste(sampledgtrees.fpathbase, g, ".txt", sep="")
    catln(2, "<logTree", numbered.id("", g, ".treeFileLog"), namevalue("logEvery", BEASTXML.gtrees.logevery), 
              namevalue("fileName", fname), 
              namevalue("sortTranslationTable", "true"),
              namevalue("nexusFormat", "true"), ">") 
    catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
    catln(2, "</logTree>")
    }
  
  commentln("Species tree log file.")
  catln(2, "<logTree", id("spptreeFileLog"), namevalue("logEvery", BEASTXML.spptree.logevery), 
         namevalue("fileName", sampledspptrees.fpath), 
         namevalue("sortTranslationTable", "true"),
         namevalue("nexusFormat", "true"), ">") 
  catln(3, idref("speciesTree", "sptree"))  
  catln(2, "</logTree>")    
  } 


  
 
make.mcmc <- function(nofgenes, taxa, beast.priors, sampledgtrees.fpathbase,
                   sampledspptrees.fpath, sampledparams.fpath, DBUGTUNE.fpath) { 
  catln(1, "<mcmc", id("mcmc"), namevalue("chainLength", BEASTXML.chain.length),
                       namevalue("autoOptimize", "true") , ">")
  catln(2, "<posterior", id("posterior"), namevalue("threads", 1), ">")
  catln(3, "<prior", id("prior"), ">") 
  catln(4, idref("speciesCoalescent", "species.coalescent"))
  catln(4, idref("mixedDistributionLikelihood", "species.popSize"))
  catln(4, idref("speciationLikelihood", "speciation.likelihood"))
  
  for (g in 1:nofgenes) {
    catln(4, "<logNormalPrior", namevalue("mean", 1.0), namevalue("stdev", 1.25),
           namevalue("offset", 0),  namevalue("meanInRealSpace", "false"), ">")
    catln(5, numbered.idref("parameter", "", g, ".kappa"))
    catln(4, "</logNormalPrior>")
    }
  
  # relative clock rates prior for genes 2,3,..    
  for (g in 1:nofgenes) {
    if (g > 1) {
      catln(4, "<logNormalPrior",
            namevalue("mean", beast.priors$relativerates.meanlog),
            namevalue("stdev", beast.priors$relativerates.sdlog),
            namevalue("offset", 0),  namevalue("meanInRealSpace", "false"), ">")
      catln(5, numbered.idref("parameter", "", g, ".clock.rate"))
      catln(4, "</logNormalPrior>")
      }
    }
    
  # popmean prior
  catln(4, "<logNormalPrior", 
        namevalue("mean", beast.priors$popmean.meanlog),
        namevalue("stdev", beast.priors$popmean.sdlog),
        namevalue("offset", 0),
        namevalue("meanInRealSpace", "false"), ">")
  catln(5, idref("parameter", "species.popMean"))
  catln(4, "</logNormalPrior>")  
  
  # birth-death-collapse growth rate prior
  catln(4, "<logNormalPrior",
        namevalue("mean", beast.priors$growthrate.meanlog),
        namevalue("stdev", beast.priors$growthrate.sdlog),
        namevalue("offset", 0),
        namevalue("meanInRealSpace", "false"), ">")
  catln(5, idref("parameter", "species.birthDeathCollapse.meanGrowthRate"))
  catln(4, "</logNormalPrior>") 
  
  # birth-death-collapse relative death rate prior (always uniform)
  catln(4, "<betaPrior", namevalue("shape", 1.0), namevalue("shapeB", 1.0), ">")
  catln(5, idref("parameter", "species.birthDeathCollapse.relativeDeathRate"))
  catln(4, "</betaPrior>") 

  # birth-death-collapse collapse weight prior
  catln(4, "<betaPrior",
        namevalue("shape", beast.priors$collapseweight.shape),
        namevalue("shapeB", beast.priors$collapseweight.shapeB), ">")
  catln(5, idref("parameter", "species.birthDeathCollapse.collapseWeight"))
  catln(4, "</betaPrior>") 
    
  catln(3, "</prior>")
  
  catln(3, "<likelihood", id("likelihood"), ">")
  for (g in 1:nofgenes) {
    catln(4, numbered.idref("treeLikelihood", "", g, ".treeLikelihood"))
    }
  catln(3, "</likelihood>")
  catln(2, "</posterior>")
  
  catln(2, idref("operators", "operators"))
  
  make.logs(nofgenes, taxa, sampledgtrees.fpathbase, sampledspptrees.fpath, 
                      sampledparams.fpath, DBUGTUNE.fpath)

  catln(1, "</mcmc>")
  }
  
  





