

#############################################################
#####         low level utility routines       ##############
#############################################################
  
namevalue <- function(nam, val) {
  paste(nam, "=\"", val, "\"", sep="") 
  }

numbered.id <- function(before, g, after) {
  paste("id=\"", before, g, after, "\"", sep="") 
  }
  
id <- function(id) { 
  paste("id=\"", id, "\"", sep="")
  }
  
  
idref <- function(element, id) {
  paste("<", element, " idref=\"", id, "\" />", sep="") 
  }  

numbered.idref <- function(element, before, g, after) {
  paste("<", element, " idref=\"", before, g, after, "\" />", sep="") 
  }  
  

catln <- function(indent, ...) {
  if (indent == 0) {
    cat(..., "\n", sep="")
    } else {
    tabs <- paste(rep("\t", indent), sep="", collapse="")
    cat(tabs, ..., "\n") 
    }
  }
 
commentln <- function(txt) {
cat("<!-- ", txt, " -->\n"); 
}


  
#############################################################
##########           models and priors         ##############
#############################################################
  

make.patterns <- function(g) {
  catln(1, "<patterns", numbered.id("", g, ".patterns"), namevalue("from", 1), ">")
  catln(2, numbered.idref("alignment", "alignment", g, ""))
  catln(1, "</patterns>")
  }
  
make.constantSize <- function() { 
 	catln(1, "<constantSize",  id("constant"), namevalue("units", "substitutions"), ">")
  catln(2, "<populationSize>")
  catln(3, "<parameter",  id("constant.popSize"),  namevalue("value", 0.016),
                 namevalue("lower", 0.0), namevalue("upper", "Infinity"), "/>")
  catln(2, "</populationSize>")
  catln(1, "</constantSize>")
 }
  
make.coalescentTree <- function(g, roothgt) {
  catln(1, "<coalescentTree ", numbered.id("", g, ".startingTree"),
          namevalue("rootHeight", roothgt), ">")
  catln(2, idref("taxa", "taxa"))
  catln(2, idref("constantSize", "constant"))
  catln(1, "</coalescentTree>")
  }


make.treeModel <- function(g) {
  catln(1, "<treeModel", numbered.id("", g, ".treeModel"), ">")
  catln(2, numbered.idref("coalescentTree", "", g, ".startingTree"))
  
  catln(2, "<rootHeight>")
  catln(3, "<parameter", numbered.id("", g, ".treeModel.rootHeight"), "/>")
  catln(2, "</rootHeight>")
  
  catln(2, "<nodeHeights internalNodes=\"true\" >")
  catln(3, "<parameter", numbered.id("", g, ".treeModel.internalNodeHeights"), "/>")
  catln(2, "</nodeHeights>")

  catln(2, "<nodeHeights internalNodes=\"true\" rootNode=\"true\" >")
  catln(3, "<parameter", numbered.id("", g, ".treeModel.allInternalNodeHeights"), "/>")
  catln(2, "</nodeHeights>")
  
  catln(1, "</treeModel>")
  }



make.strictClockBranchRate <- function(g) {
  catln(1, "<strictClockBranchRates", numbered.id("", g, ".branchRates"), ">")
  catln(2, "<rate>")
  catln(3, "<parameter", numbered.id("", g, ".clock.rate"), 
           namevalue("value", 1.0), namevalue("lower", 0.0),
           namevalue("upper", "Infinity"), "/>") 
  catln(2, "</rate>")
  catln(1, "</strictClockBranchRates>")
  }


  
make.HKYModel <- function(g) {
  catln(1, "<HKYModel", numbered.id("", g, ".hky"), ">")
  catln(2, "<frequencies>")
  catln(3, "<frequencyModel", namevalue("dataType", "nucleotide"), ">")
  catln(4, "<frequencies>")
  catln(5, "<parameter", numbered.id("", g, ".frequencies"), 
                 namevalue("value", "0.25 0.25 0.25 0.25"), "/>")
  catln(4, "</frequencies>")
  catln(3, "</frequencyModel>")
  catln(2, "</frequencies>")
  catln(2, "<kappa>")
  catln(3, "<parameter", numbered.id("", g, ".kappa"), namevalue("value", 2.0),
              namevalue("lower", 0), namevalue("upper", "Infinity"), "/>")
  catln(2, "</kappa>")
  catln(1, "</HKYModel>")
  }


make.siteModel <- function(g) {
  catln(1, "<siteModel", numbered.id("", g, ".siteModel"), ">")
  catln(2, "<substitutionModel>")
  catln(3, numbered.idref("HKYModel", "", g, ".hky"))
  catln(2, "</substitutionModel>")
  catln(1, "</siteModel>")
  }


make.treeLikelihood <- function(g) {
  catln(1, "<treeLikelihood", numbered.id("", g, ".treeLikelihood"), 
                namevalue("useAmbiguities", "false"), ">")
  catln(2, numbered.idref("patterns", "", g, ".patterns"))
  catln(2, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, numbered.idref("siteModel", "", g, ".siteModel"))
  catln(2, numbered.idref("strictClockBranchRates", "", g, ".branchRates"))
  catln(1, "</treeLikelihood>")
  }


################################################################################
########################      Species  tree         ############################
################################################################################


make.SpeciesBindings <- function(nofgenes, taxa) {
  catln(1, "<species", id("species"), ">")
  for (i in 1:length(taxa)) {
    catln(2, "<sp", id(paste(taxa[i], "sp", sep="_")), ">")
    catln(3, idref("taxon", taxa[i]))
    catln(2, "</sp>")
    }
  catln(2, "<geneTrees", id("geneTrees"), ">")
  for (g in 1:nofgenes) {
    catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
    }
  catln(2, "</geneTrees>") 
  catln(1, "</species>")
  }

  
make.speciesTree <- function(initpopsize) {
  catln(1, "<speciesTree", id("sptree"), namevalue("constantRoot", "true"), ">")
  catln(2, idref("species","species"))
  catln(2, "<sppSplitPopulations", namevalue("value", initpopsize), ">")
  catln(3, "<parameter", id("speciesTree.splitPopSize"), "/>")
  catln(2, "</sppSplitPopulations>")
  catln(1, "</speciesTree>")
}


make.birthDeathCollapseModel <- function() {
  catln(1, "<birthDeathCollapseModel",  id("birthDeathCollapse"), 
           namevalue("units","substitutions"), namevalue("collapseHeight", BEASTXML.collapseheight), ">")
  catln(2, "<speciesTree>")
  catln(3, idref("speciesTree", "sptree"))
  catln(2, "</speciesTree>")
  
  catln(2, "<birthMinusDeathRate>")
  catln(3, "<parameter", id("species.birthDeathCollapse.meanGrowthRate"), 
             namevalue("value","100"), namevalue("lower", "0.0"), 
             namevalue("upper", "Infinity"), "/>")  
  catln(2, "</birthMinusDeathRate>")    
         
  catln(2, "<relativeDeathRate>")
  catln(3, "<parameter", id("species.birthDeathCollapse.relativeDeathRate"), 
             namevalue("value","0.5"), namevalue("lower", "0.0"), 
             namevalue("upper", "1.0"), "/>")  
  catln(2, "</relativeDeathRate>") 
  
  
  catln(2, "<originHeight>")
  catln(3, "<parameter", id("species.birthDeathCollapse.originHeight"), 
             namevalue("value","0.2"), namevalue("lower", "0.0"), 
             namevalue("upper", "Infinity"), "/>")  
  catln(2, "</originHeight>") 
  
  
  catln(2, "<collapseWeight>")
  catln(3, "<parameter", id("species.birthDeathCollapse.collapseWeight"), 
             namevalue("value","0.5"), namevalue("lower", "0.0"), 
             namevalue("upper", "1.0"), "/>")  
  catln(2, "</collapseWeight>") 

  catln(1, "</birthDeathCollapseModel>")
}


make.speciationLikelihood <- function() {
  catln(1, "<speciationLikelihood", id("speciation.likelihood"), ">")
  catln(2, "<model>")
  catln(3, idref("birthDeathCollapseModel", "birthDeathCollapse"))
  catln(2, "</model>")
  catln(2, "<speciesTree>")
  catln(3, idref("speciesTree", "sptree"))
  catln(2, "</speciesTree>")  
  catln(1, "</speciationLikelihood>")
}


make.speciesCoalescent <- function() {
  catln(1, "<speciesCoalescent", id("species.coalescent"), ">")
  catln(2, idref("species", "species"))
  catln(2, idref("speciesTree", "sptree"))  
  catln(1, "</speciesCoalescent>")
}



make.gammaDistributionModel <- function(shape, initpopval, first) {
  catln(3, "<gammaDistributionModel>")
  catln(4, "<shape>")
  catln(5, shape)
  catln(4, "</shape>")
  catln(4, "<scale>")
  if (first) {
    catln(5, "<parameter", id("species.popMean"), namevalue("value", initpopval), 
               namevalue("lower", 0.0), namevalue("upper", "Infinity"),"/>")
    } else {
    catln(5, idref("parameter", "species.popMean"))
    }
  catln(4, "</scale>")
  catln(3, "</gammaDistributionModel>")
  }


make.mixedDistributionLikelihood <- function(nofspp, initpopval) { 
flags <- ""
for (i in 1:nofspp) {
  flags <- paste(flags, "1")
  }
for (i in 1:(2*nofspp - 2)) {
  flags <- paste(flags, "0")
 }
catln(1, "<mixedDistributionLikelihood", id("species.popSize"), ">")
catln(2, "<distribution0>")
make.gammaDistributionModel(2, initpopval, TRUE)
catln(2, "</distribution0>")
catln(2, "<distribution1>")
make.gammaDistributionModel(4, initpopval, FALSE)
catln(2, "</distribution1>")
catln(2, "<data>")
catln(3, idref("parameter", "speciesTree.splitPopSize"))
catln(2, "</data>")
catln(2, "<indicators>")
catln(3, "<parameter",  namevalue("value", flags), "/>")
catln(2, "</indicators>")  
catln(1, "</mixedDistributionLikelihood>")
}
         
         
         
         
#############################################################
###########        Statistics       #########################
#############################################################

make.NClustersStatistic <- function() {
commentln("Statistic for the number of clusters")
catln(1, "<bdcNClustersStatistic", id("nClusters"), namevalue("name", "nClusters"), ">")
catln(2, "<collapseModel>")
catln(3, idref("birthDeathCollapseModel", "birthDeathCollapse"))
catln(2, "</collapseModel>")
catln(2, "<speciesTree>")
catln(3, idref("speciesTree", "sptree"))
catln(2, "</speciesTree>")
catln(1, "</bdcNClustersStatistic>")
}


trmca.idname <- function(letter) {
if (nchar(letter)==0) {
  idname = "species.tree.root"
  } else {
  idname = paste("truespecies.", letter, sep="")
  }
idname 
}


make.trmcaStatistic <- function(taxasubset, letter) {
if (nchar(letter)==0) {
  commentln("TMRCA Statistic for the species tree")
  } else {
  commentln(paste("TMRCA Statistic for species ", letter))
  }  
idname <- trmca.idname(letter)  
catln(1, "<tmrcaStatistic", id(idname), namevalue("name", idname), ">") 
catln(2, idref("speciesTree", "sptree"))
catln(2, "<mrca>")
catln(3, "<taxa>")
for (tx in taxasubset) {
  catln(4, idref("sp", paste(tx, "sp", sep="_")))
  }
catln(3, "</taxa>")
catln(2, "</mrca>")
catln(1, "</tmrcaStatistic>")
}

                                         

#############################################################
##########            MCMC operators           ##############
#############################################################
 


make.general.numbered.scaleOperator <- function(g, sfactor, weight, before, after) {
  catln(2, "<scaleOperator", namevalue("scaleFactor", sfactor),
         namevalue("weight", weight), ">")
  catln(3, numbered.idref("parameter", before, g, after))
  catln(2, "</scaleOperator>")
  }		
		
make.general.scaleOperator <- function(sfactor, weight, param) {
  catln(2, "<scaleOperator", namevalue("scaleFactor", sfactor),
         namevalue("weight", weight), ">")
  catln(3, idref("parameter", param))
  catln(2, "</scaleOperator>")
  }			
		
		
make.scaleOperator.kappa <- function(g, weight) {
  make.general.numbered.scaleOperator(g, 0.75, weight, "", ".kappa")
  }
  

make.scaleOperator.clockrate <- function(g, weight) {
  make.general.numbered.scaleOperator(g, 0.75, weight, "", ".clock.rate")
  }  
	
make.deltaExchange.frequencies <- function(g, weight) {
  catln(2, "<deltaExchange", namevalue("delta", 0.01),
         namevalue("weight", weight), ">")
  catln(3, numbered.idref("parameter", "", g, ".frequencies"))
  catln(2, "</deltaExchange>")  
  }
  

make.subtreeSlide.treeModel <- function(g, weight) {
  catln(2, "<subtreeSlide", namevalue("size", .002), 
       namevalue("gaussian", "true"), namevalue("weight", weight), ">")
  catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, "</subtreeSlide>") 
  }
  

  
make.narrowExchange.treeModel <- function(g, weight) {
  catln(2, "<narrowExchange", namevalue("weight", weight), ">")
  catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, "</narrowExchange>") 
  }
  
   
make.wideExchange.treeModel <- function(g, weight) {
  catln(2, "<wideExchange", namevalue("weight", weight), ">")
  catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, "</wideExchange>") 
  }
  
   
make.wilsonBalding.treeModel <- function(g, weight) {
  catln(2, "<wilsonBalding", namevalue("weight", weight), ">")
  catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, "</wilsonBalding>") 
  
  }
  
  
make.scaleOperator.treeModel.rootHeight <- function(g, weight) {
  make.general.numbered.scaleOperator(g, 0.75, weight, "", ".treeModel.rootHeight")
  }
  

 
make.uniformOperator.treeModel.internalNodeHeights <- function(g, weight) {
  catln(2, "<uniformOperator", namevalue("weight", weight), ">")
  catln(3, numbered.idref("parameter", "", g, ".treeModel.internalNodeHeights"))
  catln(2, "</uniformOperator>")
  }
  
   
   
make.upDownOperator.clockrate.allInternalNodeHeights <- function(g, weight) {
  catln(2, "<upDownOperator", namevalue("scaleFactor", 0.75), 
           namevalue("weight", weight), ">")
  catln(3, "<up>")
  if (g>1) {
    catln(4, numbered.idref("parameter", "", g, ".clock.rate")) 
    }  
  catln(3, "</up>")
  catln(3, "<down>")
  catln(4, numbered.idref("parameter", "", g, ".treeModel.allInternalNodeHeights"))
  catln(3, "</down>")
  catln(2, "</upDownOperator>")
  } 
  
  
      
make.upDownOperator.clocks.pops.heights <- function(nofgenes, weight) {
  catln(2, "<upDownOperator", namevalue("scaleFactor", 0.75), 
           namevalue("weight", weight), ">")
           
  catln(3, "<up>")
  for (g in 1:nofgenes) {
    if (g>1) {
      catln(4, numbered.idref("parameter", "", g, ".clock.rate")) 
    }
  }
  catln(4, idref("parameter", "species.birthDeathCollapse.meanGrowthRate")) 
  catln(3, "</up>")
  
  catln(3, "<down>")
  catln(4, idref("speciesTree", "sptree"))
  catln(4, idref("parameter" , "species.popMean"))
  catln(4, idref("parameter" , "speciesTree.splitPopSize"))

  for (g in 1:nofgenes) {
    catln(4, numbered.idref("parameter", "", g, ".treeModel.allInternalNodeHeights"))
  }
  catln(3, "</down>")
  catln(2, "</upDownOperator>")
  }
  

      
make.nodeReHeightOperator <- function(weight) {
  catln(2, "<nodeReHeight", namevalue("weight", weight), ">")
  catln(3, idref("species", "species"))
  catln(3, idref("speciesTree", "sptree"))
  catln(2, "</nodeReHeight>")
}



