#                             DISSECT.

# Started 2013-10-03. 
# This is for testing species delimitation algorithm DISSECT



library(ape)
library(phangorn)

################################################################################
###################### Setting up file paths, loading code #####################

cat("DISSECT ('GNT' version) started\n")
Args <- commandArgs(TRUE)
#Args <- c("C:/Users/Graham/AAA/Programming/tmp/dissect/C")
stopifnot(length(Args) == 1) 

# Where this code is
setwd("C:/Users/Graham/AAA/Programming/biology/SpeciesDelimSimulations")

# input config is here. All output goes in subdirectories of this
CONFIG.DIR <- Args[1]

# java classpath for calling BEAST.
java.classpath <- "."
java.classpath <- paste(java.classpath, 
     "C:\\Users\\Graham\\AAA\\Programming\\Goteborg\\beasttrunk20120509\\out\\production\\Beasttrunk20120509", sep=";")
java.classpath <- paste(java.classpath,
     "C:\\Users\\Graham\\AAA\\Programming\\Goteborg\\beasttrunk20120509\\lib\\*" ,sep=";")

# for calling Seq-Gen             
seqgenbasecmd <- "seqgen.exe -mHKY -t3.0 -f0.3,0.2,0.2,0.3"

 
# Code providing functions
source("DISSECT_0randomseeds.r")
source("DISSECT_1readconfig-pparams.r")
source("DISSECT_1readconfig-GNT.r")
source("DISSECT_2makespptree.r")
source("DISSECT_3makegtrees.r")
source("DISSECT_4makeseqfiles.r")
source("DISSECT_5beastxml_toplevel.r")
source("DISSECT_6beastxml_bits.r")
source("DISSECT_7Simmatrix.r")



################################################################################
######################### Top level functions ##################################


make.genetrees.for.one.GNT.one.replicate <- function(config, spptree, i, r) { 
  # Simulate the gene trees
  for (g in 1:config$configG[i]) {
    gt <- make.one.genetree.as.Newick(spptree, config$configT[i], getrandomseed(r, g))
    sink(fpath.gtree(config, i, r, g))
    cat(gt)
    sink(NULL)
    pdf(file=fpath.gtree.pdf(config, i, r, g))
    phylo.tree <- read.tree(text=gt)
    plot.phylo(phylo.tree, edge.color="grey")
    axisPhylo()
    title(paste("Simulated gene tree: rep", r, "gene", g))
    dev.off()
    }
  }




make.sequences.for.one.GNT.one.replicate <- function(config, i, r) { 
  # Simulate the sequence alignments
  for (g in 1:config$configG[i]) {  
    SeqGen.cmd <- paste(seqgenbasecmd, 
                        " -l", config$seqs$len, 
                        " -n1 -z", getrandomseed(r, g), 
                        " ", fpath.gtree(config, i, r, g), 
                        " -y ", fpath.seq(config, i, r, g), 
                        sep="")
    cat("Running seqgen with command:", SeqGen.cmd, "\n")
    system(SeqGen.cmd)
    }
  }
  
  
make.upgma.gtrees.for.one.GNT.one.replicate <- function(config, i, r) { 
  for (g in 1:config$configG[i]) {               
    gseq <- read.dna(file=fpath.seq(config, i, r, g), format = "sequential")
    gmat <- dist.dna(gseq, as.matrix=TRUE)
    upgma.gtree <- upgma(gmat)
    pdf(file=fpath.upgmagtree.pdf(config, i, r, g), width=9, height=7)
    plot.phylo(upgma.gtree)
    title(main=paste("UPGMA gene tree", g, "GNT", i, "rep", r))
    dev.off()
    }
  }


make.beastxml.for.one.GNT.one.replicate <- function(config, spptree, i, r) {  
   make.genetrees.for.one.GNT.one.replicate(config, spptree, i, r)
   make.sequences.for.one.GNT.one.replicate(config, i, r) 

  # Make BEAST XML
  sink(fpath.beastxml(config, i, r))
  make.beastxml.from.sim.spptree(config$configG[i],
                collect.taxa(spptree),
                config$progparams$beast.priors,
                fpath.seqs.fpathbase(config, i, r), 
                fpath.sampledgtrees.fpathbase(config, i, r),
                fpath.sampledspptrees(config, i, r), 
                fpath.sampledparams(config, i, r), 
                fpath.DBUGTUNE(config, i, r))
  sink(NULL)
  }





run.beast.for.one.GNT.one.replicate <- function(config, i, r) {
  if (config$progparams$beastseed > 0) {
    randseed <- config$progparams$beastseed
    } else {
    randseed <- 1000*i + r
    }
  BEAST.cmd <- paste("java -enableassertions -classpath", java.classpath, 
          "dr.app.beast.BeastMain -seed", randseed, "-overwrite -working", 
          fpath.beastxml(config, i, r))
  cat(BEAST.cmd, "\n")
  errorcode <- system(BEAST.cmd)
  stopifnot(errorcode == 0)  # don't seem to work
  }
  
  
  
  
run.treeannotator.for.one.GNT.one.replicate <- function(config, i, r) {
  for (g in 1:config$configG[i]) {
    TreeAnnotator.cmd <- paste("java -classpath", java.classpath, 
          "dr.app.tools.TreeAnnotator -burnin", config$progparams$treeann.gtrees.burnin, 
          fpath.sampledgtree(config, i, r, g),
          fpath.mccgtree(config, i, r, g)
          )
    system(TreeAnnotator.cmd)
    }
  TreeAnnotator.cmd <- paste("java -classpath", java.classpath, 
          "dr.app.tools.TreeAnnotator -burnin", config$progparams$treeann.stree.burnin,
          fpath.sampledspptrees(config, i, r),
          fpath.mccspptree(config, i, r)
          )      
   system(TreeAnnotator.cmd)
  }  
  
    
run.speciesdelimanalyser.for.one.GNT.one.replicate <- function(config, i, r) {
  SpeciesDelimitationAnalyser.cmd <- paste("java -classpath", java.classpath, 
          "dr.app.tools.SpeciesDelimitationAnalyser -burnin", config$progparams$sppdelimanal.burnin,
          "-collapseheight", config$progparams$sppdelimanal.collapseheight,
          "-simcutoff", config$progparams$sppdelimanal.simcutoff,
          fpath.sampledspptrees(config, i, r),
          fpath.clusterings(config, i, r)
          )      
   system(SpeciesDelimitationAnalyser.cmd)
   cat("\n")
  }
  
  
make.simmatrix.for.one.GNT.one.replicate <- function(config, i, r) {
  make.simmatrix(fpath.clusterings(config, i, r),
      fpath.simmatrix(config, i, r), fpath.simmatrix.pdf(config, i, r)) 
} 
  
  
################################################################################
########################## Business starts here ################################


cat("DISSECT simulation running with directory", CONFIG.DIR, "\n")

CONFIG <- read.config()

cat("Config has beeen read\n")

nofGNT <- length(CONFIG$configG)
BEASTXML.collapseheight <- CONFIG$progparams$beast.collapseheight
BEASTXML.chain.length <- CONFIG$progparams$beastchainlength
BEASTXML.screen.logevery <- CONFIG$progparams$beastscreen.logevery
BEASTXML.params.logevery <- CONFIG$progparams$beastparams.logevery
BEASTXML.gtrees.logevery <- CONFIG$progparams$beastgtrees.logevery
BEASTXML.spptree.logevery <- CONFIG$progparams$beastspptree.logevery
BEASTXML.dbugtune.logevery <- CONFIG$progparams$beastdbugtune.logevery





replicate.sets <- NULL
for (i in 1:nofGNT) {
  R <- CONFIG$configR[i]
  replicate.sets <- c(replicate.sets, list(1:R))
  }



for (i in 1:nofGNT) {
  dir.create(fpath.GNTdpath(CONFIG, i))
  
  # Make the species tree
  spptree <- make.spptree(CONFIG$spptree)
  cat("GNT loop: i =", i, "made species tree\n")
  # add nlins at tips
  for (nn in 1:length(spptree)) {
    if (spptree[[nn]]$nheight == 0) {
      spptree[[nn]]$nlins <- CONFIG$configN[i]
      } else {
      spptree[[nn]]$nlins <- -1
      }
    }  
  Newick.spptree <- spptree.to.Newick(spptree)
  print(Newick.spptree)
  phylo.spptree <- read.tree(text=Newick.spptree)                                                                                          
  
  pdf(file=fpath.spptree.pdf(CONFIG, i))
  plot.phylo(phylo.spptree)
  axisPhylo()
  title("Simulated species tree")
  dev.off()
  
  sink(fpath.spptree(CONFIG, i))
  print.spptree(spptree)
  sink(NULL)

  for (r in replicate.sets[[i]]) {
    make.beastxml.for.one.GNT.one.replicate(CONFIG, spptree, i, r)
    }
  }


for (i in 1:nofGNT) {
  for (r in replicate.sets[[i]]) {
    make.upgma.gtrees.for.one.GNT.one.replicate(CONFIG, i, r)
    }
  }


for (i in 1:nofGNT) {
  for (r in replicate.sets[[i]]) {
    run.beast.for.one.GNT.one.replicate(CONFIG, i, r)
    }
  }


for (i in 1:nofGNT) {
  for (r in replicate.sets[[i]]) {
    run.treeannotator.for.one.GNT.one.replicate(CONFIG, i, r)
    }
  }
  
for (i in 1:nofGNT) {
  for (r in replicate.sets[[i]]) {
    run.speciesdelimanalyser.for.one.GNT.one.replicate(CONFIG, i, r)
    }
  }
  
for (i in 1:nofGNT) {
  for (r in replicate.sets[[i]]) {
    make.simmatrix.for.one.GNT.one.replicate(CONFIG, i, r)
    }
  }
  

